%
%function [beta,sterr,error2,positioner,Fvalue]=interactPVAR(ydata,xdata,I,lag,Idata,nexog,restr,timing)
%
%=====================================================================================
% Program produces structural beta corefficents of a Panel VAR which allows for
% interaction terms. It allows interaction terms but imposes a Choleski structure.
% Hence, the data needs to be entered according to the ordering.
%
% Inputs are:
% - ydata:   The endogenous variables
% - xdata:   Identical to ydata unless the user specified contemporaneous contraints
%            which excluded observation from the LHS variables but not from the RHS
%            variables, e.g. exclude observation if today's inflation is >100%
% - I:       Number of cross-sectional units (e.g. number of countries)
% - lag:     Number of lags to be used in the estimation
% - Idata:   Interaction Terms used inthe estimation. A matrix of dimnesion
%            [I*time - by - # of interaction variables]
% - nexog:   Number of strictly exogenous variables (which have to be in the first
%            columns of the y- and x-data)
% - restr:   Is a matrix of size nvar*(nvar*nint+1) with zeros and ones defining for
%            which equation which interaction and variable a restriction is supposed
%            to be applied (ones leading to an exclusion of the interaction/or variable).
%            If there is an interaction in one period it is assumed to be relevant at all lags!
%            restr is of the following form:
%
%		         	      restr11 .    .   .   .   restn1
%			            .       .    .   .   .
%			   restr =  .       .    r   .   .
%     	      		.       .    .   .   .
%			           	rest1i  .    .   .   .   restni
%
%            Example a 2*2 VAR with two iteraction terms would possibly be
%
%			   restr = rest11   rest21  =  [0   1   1]  [1   1  1]
%			           rest12   rest22     [0   0   0]  [0   1  1]
%
%            which implies that the dynamcis of the first variable are independent of the other variable
%            and the interactions (rest11 and rest21) and only depends on its own lagged values. The second
%            (endogenous) variable's response to the shock can vary with the interactions (rest12) but its
%            lagged values have always the same impact (rest22) and do not vary with the interactions.
%            Such a structure may stand for a Bi-Variate VAR of terms of trade and GDP growth, where the
%            terms of trade are only a function of their past values and the repsonse of GDP to terms of trade
%            may vary with the exchaneg rate regime (1. Interaction) and the degree of debt (2. Interaction)
%            However, the impact of lagged values of GDP on today's GDP is independent of the exchaneg rate regime
%            and the level of debt.
%
% - timing =  0 implies that interaction is always done with time t value
%   timing = -i implies that interaction is always done with time t-i value
%   timing =  1 implies that the the respective time is taken, i.e. t for the
%               intercept; t-1 for the first lag; t-2 for the second lag ...
%
% The outputs are:
% - beta:       The structural beta coefficients which are ordered in the following way
%
%		         	    0       0    0  betan1
%			            0       0    .   .
%			   beta  =  0       .    b   .
%     	      		    beta    .    .   .
%			           	beta1i  .    .  betani
%
% - sterr:      The respective standard errors
% - error2:     The structural errors
% - positioner: An indicator variable which registers which observations have been used
%               for the estimation. This matters if there are missing observation and
%               serves as input and various other programs
%
% Number of parameters for VAR with one lag, three interaction terms (interacts
% with all parameters and constant)for one equation = (1+nint)*nvar+1+nint:
% [(b+b*Ex+b*Inv+b*Ex*Inv) (b+b*Ex+b*Inv+b*Ex*Inv) (b+b*Ex+b*Inv+b*Ex*Inv) (c+c*Ex+c*Inv+c*Ex*Inv)]
%====================================================================================
%
% Written by Pascal Towbin and Sebastian Weber - 20.02.2009
% Adapted by Andreas Tudyka Jun 2012

function [BETAMAT,STERRMAT,ERRORMAT,positioner,responses]=interactPVARb(ydata,xdata,I,lag,Idata,nexog,restr,timing,values,period,nrep,nburn,gdpfractions,percentgdpIRF,shock)


%----------------------------------------------------------------------------------------------------
% MANIPULATE DATA
%---------------------------------------------------------------------------------------------------

[Ty nvar]=size(ydata);
[Tx Kx]=size(xdata);
[Ti  Ki]=size(Idata);
time = size(ydata,1)/I; % number of time periods per country (have to be equal for each country)
if time ~= floor(time)
    display('Error: The time dimension is not identical across individuals!!')
end

%---------------------------------------------------------------------------------------------------------
% CREATE INTERACTION TERMS
% ORDERED ACCORDING TO [X1 X1*I1  X1*I2 ... X1*IKi  X2 X2*I1 ...XKx*IKi]
%-----------------------------------------------------------------------------------------------------------

%-----------------------------------------------------------------------------------------------------
% CREATE REGRESSORS
%----------------------------------------------------------------------------------------------------

for i=1:lag+1
    Xn(i).X = [];
end

for i=1:lag+1               % number of lags
    for j=1:I               % number of countries
        X=[];
        for nx=1:Kx         % number of included RHS variables
            Xh=[];
            for nint=1:Ki   % number of interaction terms
                if timing==1 % if respective time for interaction terms
                    start = i+(j-1)*time; % start at 1,t+1,2t+1,...,2,t+2,2t+2,...
                    ende  = j*time-lag-1+i; % end at t-lag,2t-lag,...,t-lag+1,2t-lag+1,...
                else        % if fixed lag for interaction terms
                    start = 1+lag+(j-1)*time+timing; % start at 1+lag+timing,1+lag+t+timing,...,1+lag+2t+timing,...
                    ende  = j*time+timing;           % end at   t+timing,2t+timing,3t+timing
                end
                % First interacted RHS variables
                if i==lag+1
                    Xh=[Xh ydata(i+(j-1)*time:j*time-lag-1+i,nx).*Idata(start:ende,nint)]; % contemporaneous part in RHS variables will be restricted (if restrictions are present at all)
                else
                    Xh=[Xh xdata(i+(j-1)*time:j*time-lag-1+i,nx).*Idata(start:ende,nint)]; % else use unrestricted RHS variables
                end
            end % end loop over interaction terms nint
            % Then concatenate such that non-interacted term is left of
            % interacted variable e.g.[y y*int]
            if i==lag+1
                Xh=[ydata(i+(j-1)*time:j*time-lag-1+i,nx) Xh]; % last part of regressor matrices will be restriceted contemporaneous variables
            else
                Xh=[xdata(i+(j-1)*time:j*time-lag-1+i,nx) Xh];
            end
            X=[X Xh]; % Now concatenate for each variable so that you get e.g. [(y y*int) (i i*int)...]
        end % end loop over RHS variables
        Xhelp(j).X(:,:) = X; % this is the RHS variables matrix for one country over all interaction terms for one lag
        Xn(i).X = [Xn(i).X   % now stack all countries for one lag
            Xhelp(j).X];
        clear X
    end % end loop over countries
    XX(i).X = Xn(i).X; % each of the XXs contains one lag (+contemporanous "lag") of interaction terms for all countries
    clear Xhelp(j).X   Xn(i).X
end % end loop over lags

%----------------------------------------------
% Create intercept
X = ones(size(XX(1).X,1),1);
for i=1:lag+1
    X = [XX(i).X X];
end
%-----------------------------------------------------------------------------------------------
% CREATE DEPENDENT VARIABLES
%----------------------------------------------------------------------------------------------

% Adjust dependent series for lag numbers, i.e. for each country j shorten
% series by taking away the first 'lag' observations. Then stack back on
% top of each other to get shortened series.
for k=1:nvar
    var(k).Y = ydata(:,k);
    Y(k).Y = [];
    for j=1:I
        Yhelp = var(k).Y(1+lag+(j-1)*time:j*time);
        Y(k).Y = [Y(k).Y
            Yhelp];
        clear Yhelp
    end
end

%---------------------------------------------------------------------------------------
% ADJUSTING FOR NaN
%---------------------------------------------------------------------------------------
adjsize = size(Y(1).Y,1)/I;
tempX=[];
tempY=[];
positioner=[0 0];

for kk=1:I % do below for each country separately, i.e. loop over all countries
    
    for i=1:nvar;
        cntryY(i).Y = Y(i).Y(1+(kk-1)*adjsize:adjsize*kk,:); % each cntryY(i) contains one series of the endogenous variables form 1 to adjusted t (for one country)
    end
    cntryX = X(1+(kk-1)*adjsize:adjsize*kk,:); % cntryX contains all lags of the RHS variables from 1 to adjusted t (for one country)
    
    %------------------------------------------------------------------------------------
    % ADJUSTING FOR NaN
    %---------------------------------------------------------------------------------------
    for i=1:nvar;
        helpY(:,i)=cntryY(i).Y; % stack all LHS series for one country next to each other
    end
    
    for j=1:size(cntryX,2); % over number of RHS variables
        [ROWIND COLIND] = find(isfinite(cntryX(:,j))); % find NaNs for RHS variables
        for k=1:size(ROWIND,1);
            newX(k,:)=cntryX(ROWIND(k,1),:); % extract rows of X that are not NaN in respective column
            newY(k,:)=helpY(ROWIND(k,1),:);  % extract corresponding rows of Y
        end
        clear cntryX helpY;
        if size(ROWIND,1)>0 % if there's at least one non-NaN in column
            allnan=0;
            cntryX=newX;
            helpY=newY;
            clear newX newY;
        else
            allnan=1; % if all entries in column of X are NaN
            break
        end
        clear ROWIND COLIND
    end
    
    if allnan==1; % if all entries in column of X are NaN
        if kk==1; % country == 1
            positioner(kk,1) = 0;
            positioner(kk,2) = 0;
        else
            positioner(kk,1)=positioner(kk-1,2);
            positioner(kk,2)=positioner(kk-1,2);
        end
        continue
    else
    end
    
    % Now non-NaN rows of X and corresponding rows of Y extracted. The helpY
    % rows might still be NaN though which cannot be used for regression.
    for j=1:size(helpY,2)
        [ROWIND2 COLIND2] = find(isfinite(helpY(:,j)));
        for k=1:size(ROWIND2,1)
            newX(k,:)=cntryX(ROWIND2(k,1),:); % extract corresponding rows of X in which Y in not NaN in respective column
            newY(k,:)=helpY(ROWIND2(k,1),:);  % extract rows Y in which it is not NaN in respective column
        end
        clear cntryX helpY;
        
        if size(ROWIND2,1)>0
            allnan=0;
            cntryX=newX;
            helpY=newY;
            clear newX newY;
        else
            allnan=1;
            break
        end
        clear ROWIND2 COLIND2
    end
       
    if allnan==0
        clear cntryY
        for i=1:nvar
            cntryY(i).Y = helpY(:,i);
        end
        clear helpY
        
        if kk==1
            positioner(kk,1) = 1;
        else
            positioner(kk,1) = positioner(kk-1,2)+1;
        end
        positioner(kk,2) = positioner(kk,1)+size(cntryX,1)-1;
        
        for i=1:nvar
            temp2Y(:,i) = cntryY(i).Y;
        end
        tempY = [tempY
            temp2Y];
        clear helpY temp2Y
        tempX= [tempX
            cntryX];
        
        clear cntrX cntryY
    else
        if kk==1
            positioner(kk,1) = 0;
            positioner(kk,2) = 0;
        else
            positioner(kk,1)=positioner(kk-1,2);positioner(kk,2)=positioner(kk-1,2);
        end
    end
end

for i=1:nvar
    Y(i).Y=tempY(:,i);
end

clear tempY
X = tempX;
clear tempX

% Taking away the lag zero part
Idata_adj=X(:,1:nvar*(Ki+1)); % lag zero part
X=X(:,1+nvar*(Ki+1):end); % rest
%-----------------------------------------------------------------------------------------------------
% DEFINE RESTRICTIONS
%---------------------------------------------------------------------------------------------------
int=Ki;
restr_new=restr;

for k=1:nvar
    if k>=2
        r(k).r=zeros(sum(restr_new(k,1:(k-1)*(nint+1))) + sum(restr_new(k,:))*lag,1);
        R(k).R=zeros(sum(restr_new(k,1:(k-1)*(nint+1))) + sum(restr_new(k,:))*lag,(k-1)*(nint+1)+(nint+1)*nvar*lag+1+nint);
        rowindex=0;
        restrindex=[];
        restrindex=[kron(ones(lag,1)',(restr_new(k,1:end)))'];
        for m=1:k-1
            restrindex=[restr_new(k,1+(m-1)*(int+1):m*(int+1))' ;restrindex];
        end
        for j=1:size(restrindex,1)
            rowindex=rowindex+restrindex(j,1);
            if restrindex(j,1)==1
                R(k).R(rowindex,j)=1;
            else
            end
        end
    elseif k==1
        r(k).r=zeros(sum(restr_new(k,:))*lag,1); % get length of restriction rows times lag length to know how many restrictions per equation you need, produce vector of zeros with this length
        R(k).R=zeros(sum(restr_new(k,:))*lag,(nint+1)*nvar*lag+1+nint); % produce zero matrix: (nrest*lags)*number of total parameters per equation including constants and their interactions
        rowindex=0;
        restrindex=[];
        restrindex=[kron(ones(lag,1)',(restr_new(k,1:end)))']; % replicate in columns for each lag the first row of restriction amtrix
        for j=1:size(restrindex,1)
            rowindex=rowindex+restrindex(j,1);
            if restrindex(j,1)==1
                R(k).R(rowindex,j)=1;
            else
            end
        end
    end
end

% Making no interecept variation for the exogenous variables:
if nexog>0
    for k=1:nexog
        r(k).r=[r(k).r; zeros(nint,1)];
        R(k).R=[R(k).R; zeros(nint,size(R(k).R,2))];
        for j=1:nint
            R(k).R(size(R(k).R,1)+1-j,size(R(k).R,2)+1-j)=1;
        end
    end
end
%==========================================================================
% Estimation Bayesian style:
% ---------------------------
[BETAMAT,STERRMAT,ERRORMAT,R,responses]= bayesianest(X,Y,Idata_adj,nvar,nint,lag,values,period,nrep,nburn,gdpfractions,percentgdpIRF,shock);
