%
%function [betairf Pkomega azero]=irfpreppvar(betaeval,fPkomega,lag)
%
%==============================================================================
% Returns the covariance equivalence and reduced form betas from betas which have
% been produced by the function interactPVAR.m and afterwards evaluated with evalpvar.m.
% The function produces reduced form betas as in a simple PVAR such that all usual programs
% (in particular IRFcreator.m) can be used.
%
% Required inputs are:
% - betaeval: The structural betas which have been evaluated for a particular value of the
%             interaction terms (betaeval is the output of the evalpvar.m program)
% - fPkomega: is the (co)variance matrix generated by the structural errors from the
%             interactPVAR.m program (the off-diagonal elements are zero).
% - lag:      The number of lags used inthe estimation
%
% The ouput is given by:
% - betairf: Reduced form beta equivalents
% - Pkomega: Pkomega equivalent that derives form choleski decomposing the covariance matrix
% - azero:   The azero matrix (lower diagonal)
%
%==============================================================================
% See also evalpvar, IRFcreator, IPVARboot, IRF_IPVAR, interactPVAR
%
% Written by Pascal Towbin and Sebastian Weber - 20.02.2009
% edited 21.5.2009

function [betairf Pkomega azero]=irfpreppvar(betaeval,fPkomega,lag)

neq=size(betaeval,2);		 % number of equations
std=sqrt(diag(fPkomega));    % structural error standard deviations
% These are the contemporaneous parameters below a row of zeros:
evalbeta2=[zeros(1,neq)
    betaeval(1:neq-1,:)];
evalbeta2=evalbeta2';
% Reverse order of parameter columns and change signs.
for i=1:neq;
    evalbeta4(:,i)=evalbeta2(:,end-i+1)*(-1);
end
% This corresponds to the bold A matrix in Luetkepohl
azero=eye(neq)+evalbeta4;
% get the inverse of the reduced form error covariance cholesky decomposition by dividing bold A by respective structural error variances
for i=1:neq
    B0(i,:)=azero(i,:)./std(i,1);
end
% Get final cholesky decomposition by inverting
Pkomega=B0^(-1);
% Multiply each A matrix for each lag with the azero matrix to get
% reduced form parameters
for l=1:lag
    betairf(1+neq*(l-1):+neq*l,:)=(azero^(-1)*betaeval(1+(neq-1)+neq*(l-1):(neq-1)+neq*l,:)')';
end

betairf=[betairf
    (azero^(-1)*betaeval(lag*neq+neq-1+1:end,:)')']; %constant only 1 as constant and its interaction term are already added up.
